# Gnomon

Simple event scheduling for Ruby

## Presentation

This library provides an easy-to-use event scheduler for Ruby.
This allows easy development of asynchronous time-based triggers.
Both intervals and points are available for scheduling.

## Installation

### Gemfile
```ruby
gem 'gnomon'
```

### Terminal
```bash
gem install -V gnomon
```

## Usage

### Introduction

First, a *Gnomon* scheduler should be created:

```ruby
require 'gnomon'
sched = Gnomon.new
```

### Managing the scheduler

The scheduler itself is a simple [Runify](https://rubygems.org/gems/runify) service which can be controlled via the typical methods (_startup_, _shutdown_, _restart_).

```ruby
sched.startup			# Start the Scheduler Service
sched.restart			# Re-Start the Scheduler Service
sched.shutdown			# Stop the Scheduler Service
```

Events can then be scheduled by using either the _schedule_ generic method, or one of the three shortcut methods _schedule_at_, _schedule_in_, _schedule_every_.

All four methods take an _id_ argument, which can be used to later de-schedule an event (or a group of events).

Each method also accepts a _block_, which will be executed when the event triggers.
Finally, the _*args_ argument will be passed as-is to the _block_ upon trigger.

Any time an event triggers, a new thread is created around the event's _block_, to which is passed the event's _*args_.

### Granularity

Internally, the scheduler checks the system clock at regular intervals, and compares the next event in the queue to determine whether it's time to trigger.
This means that the scheduler has an inherent "minimal time resolution" - events can not be triggered at smaller intervals than this minimum resolution.
So why not set this directly to an infinitesimal value? Simply because the scheduler would basically spend most of its time (and the host's CPU cycles) just checking the system clock.

By default, the granularity is set to *1 second*, which is fine for most usages.
For sub-second scheduling (or very wide scheduling), the granularity can be set to any value by passing it as argument to the constructor:

```ruby
require 'gnomon'
sched_ms = Gnomon.new 0.001		# Create a Scheduler with milli-second granularity
sched_h = Gnomon.new 3600		# Create a Scheduler with hour granularity
```

### Scheduling events

#### *schedule* _id_, _mode_, _mode_options_, _timespec_, _*args_, _&block_

The _mode_ argument can be one of the following:
* *:at*    -> Trigger the event _once_ at a given date/time (provided by the _timespec_ argument)
* *:in*    -> Trigger the event _once_ after a certain amount of time has passed (provided by the _timespec_ argument)
* *:every* -> Trigger the event _periodically_ at regular intervals (provided by the _timespec_ argument)

Any event scheduled in *:at* or *:in* mode will trigger only once, and is _automatically de-scheduled_ after triggering.

The shortcut methods presented below do not perform anything other than call the _schedule_ method with the correct _mode_ argument set.

#### *schedule_at* _id_, _date_time_, _*args_, _&block_

The _schedule_at_ method schedules _&block_ to be run *once*, at exactly _date_time_, which can be any string representation of a date/time.

#### *schedule_in* _id_, _time_, _*args_, _&block_

The _schedule_in_ method schedules _&block_ to be run *once*, in exactly _time_ seconds.

#### *schedule_every* _id_, _interval_, _async_, _*args_, _&block_

The _schedule_every_ method schedules _&block_ to be run *periodically*, once every _interval_ seconds.
The _async_ argument determines when the next run is scheduled for each run:
* *async = true* -> The next trigger is scheduled *as soon as the event triggers* (before running the event's _block_).
* *async = false* -> The next trigger is scheduled *as soon as the event _block_ completes*.

### De-scheduling events

Previously-scheduled events can be de-scheduled through the _deschedule_ method, by specifying the *id* used during initial scheduling of the event.
An optional _keep_running_ argument can be set to true to indicate that any "next scheduled trigger" should remain in the event queue.

## Examples

### Basic scheduling

```ruby
#!/usr/bin/env ruby

require 'gnomon'

# Create Scheduler
sched = Gnomon.new

# Events can be scheduled before running the scheduler
sched.schedule_at(nil, "April 1 2000 10:00 AM") { puts 'April fool\'s day!' }

# Start Scheduler
sched.startup

# Events can be scheduled while the scheduler is running
sched.schedule_in(nil, 5.5) { puts 'Five and half seconds have passed :)' }
sched.schedule_every(nil, 0.1, false) { puts 'Refreshing something...' }

# Do something while scheduler is running...

# Shutdown Scheduler
sched.shutdown
```

### Self-de-scheduling event

```ruby
#!/usr/bin/env ruby

require 'gnomon'

# Create Scheduler
sched = Gnomon.new

# Start Scheduler
sched.startup

# Run this every second, but five times only (have the event de-schedule itself after 5 triggers)
counter = 0
sched.schedule_every(:foo_task, 1, false) { puts "Run ##{counter}"; sched.deschedule :foo_task if counter >= 5 }

# Shutdown Scheduler
sched.shutdown
```

### Passing arguments

```ruby
#!/usr/bin/env ruby

require 'gnomon'

# Create Scheduler
sched = Gnomon.new

# Start Scheduler
sched.startup

# Events can be scheduled while the scheduler is running
sched.schedule_in(nil, 5.5, 'Hello world!') { |s| puts "Five and half seconds have passed, time to say \"#{s}\"" }
sched.schedule_in(nil, 10, :users, [:foo, :bar]) { |category, entities| puts "Refreshing #{category}: #{entities.join ', '}" }

# Do something while scheduler is running...

# Shutdown Scheduler
sched.shutdown
```

## License

The gem is available as open source under the terms of the [MIT License](http://opensource.org/licenses/MIT).
